<?php
/**
 * Get information about your models, controllers and actions in application.
 * Original code came from Vitaliy Stepanenko (mail@vitaliy.in).
 * 
 * @author Farhan Ghazali <farhan_ghazali@fgnet.com.my>
 */
class RouteMetadata extends CApplicationComponent {
   
    /**
     * Get all information about application
     * If there are modules of your application have controllers with same name, 
     * it will raise fatal error
     * 
     * @return array 
     */
    public function getAll()
    {
        $meta=array(
            'models'=>$this->getModels(),
            'controllers'=>$this->getControllers(),
            'modules'=>$this->getModules(),
        );
        foreach ($meta['controllers'] as &$controller)
        {
            $controller=array(
                'name'=>$controller,
                'actions'=>$this->getActions($controller)
            );   
        }
        foreach ($meta['modules'] as &$module)
        {   
            $controllers=$this->getControllers($module);
                        
            foreach ($controllers as &$controller)
            {
                $controller=array(
                    'name'=>$controller,
                    'actions'=>$this->getActions($controller,$module)
                );   
            }
            
            $module=array(
                'name'=>ucfirst($module),
                'controllers'=>$controllers,
                'models'=>$this->getModels($module),
            );    
        }
        return $meta;

    }

    /**
     * Get actions of controller
     * 
     * @param  string $controller
     * @param  string $module
     * @return array 
     */
    public function getActions($controller, $module=null)
    {
        if ($module != null) {
            $path=join(DIRECTORY_SEPARATOR,array(Yii::app()->modulePath,$module,'controllers'));
            $this->_setModuleIncludePaths($module);
        } else {
            $path='protected'.DIRECTORY_SEPARATOR.'controllers';
        }        
               
        include_once($path.DIRECTORY_SEPARATOR.$controller.'.php');        
        $reflection = new ReflectionClass($controller); 
        $methods = $reflection->getMethods(); 

        $actions=array();
        foreach($methods as $method) {           
            if (strpos($method->name,'action')===0 and ctype_upper($method->name[6])) {
                $actions[]=str_replace('action','',$method->name);
            }
        }
        return $actions;
    }
  
    /**
     * Get list of controllers with actions
     * 
     * @param  string $module
     * @return array 
     */
    public function getControllersActions($module=null)
    {
        $c=$this->getControllers($module);
        foreach ($c as &$controller) {
            $controller=array(
                'name'=>$controller,
                'actions'=>$this->getActions($controller, $module)
            );   
        }
        return $c;
    }
    
    /**
     * Get list of controllers with actions
     * 
     * @param  string $module
     * @return array 
     */
    public function getControllers($module=null)
    {
        if ($module!=null) {
            $path=join(DIRECTORY_SEPARATOR,array(Yii::app()->modulePath,$module,'controllers'));            
        } else {
            $path='protected'.DIRECTORY_SEPARATOR.'controllers';
        }      
        $paths=scandir($path);
        $controllers = array_filter($this->_removeDottedDir($paths),array($this,'_isController'));
        foreach ($controllers as &$c) {    
            $c=str_ireplace('.php','',$c);
        }    
        sort($controllers);
        return $controllers;         
    }

    /**
     * Scans models directory & return array of MVC models
     * 
     * @param  string $module
     * @param  string $include_classes
     * @return array 
     */
    public function getModels($module=null,$include_classes=false)
    {

        if ($module!=null) {
            $path=join(DIRECTORY_SEPARATOR,array(Yii::app()->modulePath,$module,'models'));
        } else {
            $path='protected'.DIRECTORY_SEPARATOR.'models';
        }                

        $files = scandir($path);
        $models=array();
        foreach ($files as $f) {
            if (stripos($f,'.php')!==false) {
                $models[]=str_ireplace('.php','',$f);
                if ($include_classes) {
                    include_once($path.DIRECTORY_SEPARATOR.$f);
                } 
            }
        }       
        return $models; 
    }

    /**
     * Returns array of module names
     * 
     * @return array 
     */
    public function getModules()
    {        
        $modules=scandir(Yii::app()->modulePath);
        $modules=array_filter($modules,array($this,'_isModule'));
        $modules=$this->_removeDottedDir($modules);
        sort($modules);
        return $modules;
    }    
    
    /**
     * Used in getControllers() to filter array of files & directories
     *  
     * @see    getControllers
     * @param  string $a
     * @return boolean 
     */
    private function _isController($a)
    {
        return stripos($a,'Controller.php')!==false;
    }

    /**
     * Used in getModules() to filter array of files & directories
     * 
     * @see    getModules()
     * @param  string $a
     * @return boolean
     */
    private function _isModule($a)
    {
        return $a!='.' AND $a!='..' AND is_dir(Yii::app()->modulePath.DIRECTORY_SEPARATOR.$a);
    }

    
    /**
     * Remove hidden and dotted directory names
     * 
     * @param  $dirs array The array containing directory names.
     * @return array 
     */
    private function _removeDottedDir($dirs)
    {
        $data=array();
        foreach ($dirs as $dir) {
            if (strpos($dir, '.')!==0) {
                $data[]=$dir;
            }
        }   
        return $data;
    }
    
    /**
     * Set php include paths for module
     * 
     * @param string $module 
     */
    private function _setModuleIncludePaths($module)
    {       
        set_include_path(join(PATH_SEPARATOR,array(
            get_include_path(),
            join(DIRECTORY_SEPARATOR,array(Yii::app()->modulePath,$module,'controllers')),
            join(DIRECTORY_SEPARATOR,array(Yii::app()->modulePath,$module,'components')),
            join(DIRECTORY_SEPARATOR,array(Yii::app()->modulePath,$module,'models')),
            join(DIRECTORY_SEPARATOR,array(Yii::app()->modulePath,$module,'vendors')),
        )));
    }    
}