<?php
/**
 * FgActionAuthFilter class file.
 *
 * @author Farhan Ghazali <farhan_ghazali@fgnet.com.my>
 * @copyright Copyright &copy; 2008-2011 Mohd Farhan Afiq bin Ghazali
 */

/**
 * FgActionAuthFilter is an action based authorization filter.
 *
 * Specifically meant for MVC structure, it checks whether a user/group is authorized 
 * to access a specific module, controller and/or action.
 *
 * @author Farhan Ghazali <admin@fgnet.com.my>
 * @version 1.0
 * @package Yii
 */
class FgActionAuthFilter extends CFilter
{
    
    /**
     * The module
     * @var CModule
     */
    private $_module;
    
    /**
     * The controller
     * @var CController
     */
    private $_controller;
    
    /**
     * The action
     * @var CAction
     */
    private $_action;
    
    /**
     * The array that contains authorization/permission data
     * @var array
     */
    private $_authData;
    
    /**
     * An object that stores the currently logged in user based on Yii::app()->user->id; 
     * @var object
     */
    private $_user;
    
    /**
     * The message that user will received when they are not authorized.
     * @var string 
     */
    public $authMessage='I\'m sorry, you are not authorized to view this page.';
    
    /**
     * List of actions that do not need authentication and authorization.
     * @var array 
     */
    public $freeActions=array();
    
    /**
     * Is it enabled or not. If disabled, its free access for everybody.
     * @var integer 
     */
    public $enabled=0;
    
    /**
     * Executes filters
     * @param CFilterChain
     * @return boolean 
     */
	protected function preFilter($filterChain)
	{
        // set current controller
		$this->_controller=Yii::app()->getController();       
        $this->_module=$this->_controller->getModule();  
        $this->_action=$this->_controller->getAction();  
        
        // setup freeActions
        $freeActions=array('*'=>$this->freeActions);
                
        // get user based on username OR user id
        $criteria=new CDbCriteria;
        $criteria->condition='username=:username OR id=:id';
        $criteria->params=array(
            ':username'=>Yii::app()->user->id,
            ':id'=>Yii::app()->user->id
        );
        $this->_user=User::model()->find($criteria);
        
        // setup permissions
        if (is_null($this->_user)) {
            $permissions=$freeActions+array();
        } else {
            $permissions=(is_null($this->_user->permissions) ? array() : $this->_user->permissions)+$this->_user->group->permissions;
            $permissions=$freeActions+$permissions;
        }
        $this->_authData=$permissions;
        
        // authorize or not
        if ($this->enabled===1) {
            return $this->authorize();
        } else {
            return TRUE;
        }
	}
    
    /**
     * Checks if a user can access a specific action.
     * @param string $module Module name
     * @param string $controller Controller name
     * @param string $action Action name
     * @return boolean|CHttpException
     */
    public function authorize($module = NULL, $controller = NULL, $action = NULL)
    {        
        // get the request: module, controller, action and who is accessing it
        $module=($module===NULL && isset($this->_module)) ? ucfirst($this->_module->id) : ucfirst($module);
        $controller=(($controller===NULL) ? ucfirst($this->_controller->id) : ucfirst($controller)).'Controller';
        $action=($action===NULL) ? ucfirst($this->_action->id) : ucfirst($action);

        // get user permission data
        $permissions=$this->_authData;
        
        // when the user is authenticated
        if (!Yii::app()->user->isGuest) {            
			// by default, user is denied access
			$allowed=0;
			
            // check authorization based on group, username and actions
            if (isset($this->_module) || $module > 0) {
                if (isset($permissions[$module][$controller]) && in_array($action, $permissions[$module][$controller])) {
					$allowed=1;
                }                   
                if (isset($permissions[$module][$controller]) && in_array($action, $permissions[$module][$controller])) {
					$allowed=1;
                }
            } else {              
                if (isset($permissions[$controller]) && in_array($action, $permissions[$controller])) {
					$allowed=1;
                }                   
                if (isset($permissions[$controller]) && in_array($action, $permissions[$controller])) {
					$allowed=1;
                }            
            }
            
            // check freeActions
            if (isset($permissions['*'][$controller]) && in_array($action, $permissions['*'][$controller])) {
                $allowed=1;
            }
            
            // decide to allow access or not.
			if ($allowed==0) {
				if (Yii::app()->getRequest()->isAjaxRequest) {
                    return FALSE;
				} else {
                    throw new CHttpException(403,$this->authMessage);
				}
			} else {
				return TRUE;
			}
            
        } else {  
            // check guests
            if (isset($permissions['*'][$controller]) && in_array($action, $permissions['*'][$controller])) {
                return TRUE;
            } else {
                throw new CHttpException(403,$this->authMessage);
            }			
        }
    }	
	
}