<?php

/**
 * This is the model class for table "group".
 *
 * The followings are the available columns in table 'group':
 * @property integer $id
 * @property string $name
 * @property integer $permissions
 */
class Group extends CActiveRecord
{
	/**
	 * Returns the static model of the specified AR class.
	 * @return Group the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'group';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('name, permissions', 'required'),
            array('name', 'unique'), 
			array('name', 'length', 'max'=>32),
			// The following rule is used by search().
			// Please remove those attributes that should not be searched.
			array('id, name, permissions', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
        return array(
            'users'=>array(self::BELONGS_TO, 'User', 'group_id'),          
        );
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'name' => 'Name',
			'permissions' => 'Permissions',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
	 */
	public function search()
	{
		// Warning: Please modify the following code to remove attributes that
		// should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('name',$this->name,true);
		$criteria->compare('permissions',$this->permissions,true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

    /**
     * 
     */
    public function beforeSave()
    {
        $this->name=str_replace(' ', '', ucwords($this->name));
        $this->permissions=serialize($this->permissions);
        return parent::beforeSave();
    }
    
    /**
     * 
     */
    public function afterFind()
    {
        $this->permissions = unserialize($this->permissions);
        return parent::afterFind();
    }
    
    /**
     * Get a checkbox for a permission rule.
     * 
     * @param  string $field The name of the permission group
     * @param  string $value The action name
     * @return string 
     */
    public function getPermissionCheckBox($field,$value)
    {       
        $checked = ($this->_isPermisssionFieldSelected($field,$value)) ? 'checked="checked"' : '';
        
        $html = '<input class="permission" '.$checked.' name="'.$field.'" type="checkbox" value="'.$value.'" />';
        return $html;
    }
    
    /**
     * Check if an action is selected as allowed permission rule.
     * 
     * @param  string $field The name of the permission group
     * @param  string $value The action name
     * @return boolean 
     */
    private function _isPermisssionFieldSelected($field,$value)
    {
        if (!is_array($this->permissions)) {
            return FALSE;
        }
        
        // get access names from field name
        $field = str_replace('Group[permissions]', '', $field);
        preg_match_all('/(?<=\[)(.*?)(?=\])/', $field, $matches);
        $accessNames = array_filter($matches[0]);
        
        // seperate module and controller from $this->permissions into arrays
        foreach ($this->permissions as $key => $actions) {
            // is it a controller or a module
            if (strpos($key, 'Controller')) {
                $controllers[$key] = $actions;
            } else {
                $moreControllers = $actions;
                $modules[$key] = $moreControllers;
            }
        }      
        
        // is this fieldname a module or just a controller
        // module & controller
        if (count($accessNames) === 2) {
            $module = $accessNames[0];
            $controller = $accessNames[1];
            if (@in_array($value, $modules[$module][$controller])) {
                return TRUE;
            }        
        // just controller
        } else if (count($accessNames) === 1) {
            $controller = $accessNames[0]; 
            if (@in_array($value, $controllers[$controller])) {
                return TRUE;
            }
        }
        return FALSE;
    }    
}