<?php

/**
 * This is the model class for table "user".
 *
 * The followings are the available columns in table 'user':
 * @property integer $id
 * @property string  $username
 * @property string  $password
 * @property string  $email
 * @property string  $permissions
 * @property integer $group_id
 */
class User extends CActiveRecord
{
    public $confirmEmail;
    public $newEmail;
    public $confirmPassword;
    public $currentPassword;
    public $newPassword;
    
	/**
	 * Returns the static model of the specified AR class.
     * 
	 * @return User The static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * Returns the table name.
     * 
     * @return string The associated database table name
	 */
	public function tableName()
	{
		return 'user';
	}

	/**
	 * The definition for validation rules.
     * 
     * @return array Validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
            // required
			array('group_id', 'required'),
            array('username, email, password', 'required', 'on'=>'insert'),
            array('confirmEmail', 'required', 'on'=>'insert, changeEmail'),
            array('newEmail', 'required', 'on'=>'changeEmail'),
            array('confirmPassword', 'required', 'on'=>'insert, changePassword'),
            array('currentPassword, newPassword', 'required', 'on'=>'changePassword'),
            // data types
			array('group_id', 'numerical', 'integerOnly'=>true),
            array('email, confirmEmail, newEmail', 'email'), 
            // unique
            array('username, email', 'unique'),
            // length
			array('username', 'length', 'min'=>'4', 'max'=>128),
            array('email', 'length', 'min'=>'4', 'max'=>128),
			array('password, confirmPassword', 'length', 'min'=>'4', 'max'=>64),
            // comparison
            array('confirmEmail', 'compare', 'compareAttribute'=>'email', 'on'=>'insert'),
            array('confirmEmail', 'compare', 'compareAttribute'=>'newEmail', 'on'=>'changeEmail'),
            array('confirmPassword', 'compare', 'compareAttribute'=>'password', 'on'=>'insert'),
            array('confirmPassword', 'compare', 'compareAttribute'=>'newPassword', 'on'=>'changePassword'),
            array('newPassword', 'compare', 'compareAttribute'=>'currentPassword', 'operator'=>'!=', 'on'=>'changePassword'),             
            // custom
            array('currentPassword', 'isCurrentPassword', 'on'=>'changePassword'),            
            array('username', 'validateUsername', 'on'=>'insert'),
			// The following rule is used by search().
			// Please remove those attributes that should not be searched.
			array('id, username, email, password, group_id', 'safe', 'on'=>'search'),
		);
	}

    /**
     * Checking whether the entered password is the current user password.
     * @see rules()
     */
    public function isCurrentPassword($attribute,$params)
    {
        if($this->password!==self::encrypt($this->$attribute)) {
            $this->addError($attribute, 'This is not your current password.');
        }
    }   
    
    /**
     * Validating username to only have underscores, dots, small letters and numbers.
     * @see rules()
     */
    public function validateUsername($attribute,$params)
    {
        if(!preg_match('/^[a-z\d_.]{4,32}$/', $this->$attribute)) {
            $this->addError($attribute, 'Only small letters, numbers, underscores 
                and dots are allowed in '.$attribute.'.');
        }        
    }    
    
	/**
	 * The definition for model relationships.
     * 
     * @return array Relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
            'group'=>array(self::HAS_ONE, 'Group', 'id'),                 
		);
	}

	/**
	 * The definition of attribute labels.
     * 
     * @return array Customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id'=>'ID',
			'username'=>'Username',
			'email'=>'Email',
			'password'=>'Password',
            'permissions'=>'Permissions',
			'group_id'=>'Group',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * 
     * @return CActiveDataProvider The data provider that can return the models based on the search/filter conditions.
	 */
	public function search()
	{
		// Warning: Please modify the following code to remove attributes that
		// should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id', $this->id);
		$criteria->compare('username', $this->username,true);
		$criteria->compare('email', $this->email,true);
		$criteria->compare('password', $this->password,true);
        $criteria->compare('permissions',$this->permissions,true);
		$criteria->compare('group_id', $this->group_id);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}
    
    /**
     * Stuff to do before saving the model.
     * 
     */
    public function beforeSave()
    {
        $allowedToMd5=array('insert','changePassword');
        if (in_array($this->scenario, $allowedToMd5)) {
            if ($this->scenario=='changePassword') {
                $this->password=$this->newPassword;
            }            
            $this->password=self::encrypt($this->password);
        }  
        
        if ($this->scenario=='changeEmail') {
            $this->email=$this->newEmail;
        }         
        
        $this->username=str_replace(' ', '', strtolower($this->username));
        $this->permissions=serialize($this->permissions);
        return parent::beforeSave();
    }
    
    /**
     * 
     * 
     */
    public function afterFind()
    {
        $this->permissions = unserialize($this->permissions);
        return parent::afterFind();
    }    

    /**
     * Get a checkbox for a permission rule.
     * 
     * @param  string $field The name of the permission group
     * @param  string $value The action name
     * @return string 
     */
    public function getPermissionCheckBox($field,$value)
    {       
        $checked = ($this->_isPermisssionFieldSelected($field,$value)) ? 'checked="checked"' : '';
        
        $html = '<input class="permission" '.$checked.' name="'.$field.'" type="checkbox" value="'.$value.'" />';
        return $html;
    }
    
    /**
     * Check if an action is selected as allowed permission rule.
     * 
     * @param  string $field The name of the permission group
     * @param  string $value The action name
     * @return boolean 
     */
    private function _isPermisssionFieldSelected($field,$value)
    {
        if (!is_array($this->permissions)) {
            return FALSE;
        }
        
        // get access names from field name
        $field = str_replace('User[permissions]', '', $field);
        preg_match_all('/(?<=\[)(.*?)(?=\])/', $field, $matches);
        $accessNames = array_filter($matches[0]);
        
        // seperate module and controller from $this->permissions into arrays
        foreach ($this->permissions as $key => $actions) {
            // is it a controller or a module
            if (strpos($key, 'Controller')) {
                $controllers[$key] = $actions;
            } else {
                $moreControllers = $actions;
                $modules[$key] = $moreControllers;
            }
        }      
        
        // is this fieldname a module or just a controller
        // module & controller
        if (count($accessNames) === 2) {
            $module = $accessNames[0];
            $controller = $accessNames[1];
            if (@in_array($value, $modules[$module][$controller])) {
                return TRUE;
            }        
        // just controller
        } else if (count($accessNames) === 1) {
            $controller = $accessNames[0]; 
            if (@in_array($value, $controllers[$controller])) {
                return TRUE;
            }
        }
        return FALSE;
    }       
    
    
    /**
     * Encrypting password.
     * 
     * @return string
     */
    public static function encrypt($value) {
        return md5($value);
    }
}